@@ -25,17 +25,16 @@ BEGIN {
         require Win32;
         require Win32::Process;
         require Win32::Process::Info;
-        Win32::Process::Info->import('NT');
+        Win32::Process::Info->import();
     }
 }
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 sub _ANYPROCESS                    { return -1 }
 sub _COMMAND                       { return 0 }
 sub _DEFAULT_HOST                  { return 'localhost' }
-sub _ANY_PORT                      { return 0 }
-sub _WIN32_ERROR_SHARING_VIOLATION { return 32 }
+sub _WIN32_ERROR_SHARING_VIOLATION { return 0x20 }
 
 sub new {
     my ( $class, %parameters ) = @_;
@@ -56,7 +55,8 @@ sub new {
         push @arguments, ( '-P', $self->{profile_name} );
     }
     else {
-        my $profile_directory = $self->_setup_new_profile($parameters{profile});
+        my $profile_directory =
+          $self->_setup_new_profile( $parameters{profile} );
         push @arguments,
           ( '-profile', $profile_directory, '--no-remote', '--new-instance' );
     }
@@ -101,10 +101,18 @@ sub _launch {
 sub _launch_win32 {
     my ( $self, @arguments ) = @_;
     my $binary = $self->_binary();
-    Win32::Process::Create( my $process, "$binary",
-        'firefox ' . ( join q[ ], map { q["] . $_ . q["] } @arguments ),
-        0, Win32::Process::NORMAL_PRIORITY_CLASS(), q[.] )
-      || Carp::croak( Win32::FormatMessage( Win32::GetLastError() ) );
+    my ( $volume, $path, $name ) = File::Spec->splitpath($binary);
+    my $result =
+      Win32::Process::Create( my $process, $binary,
+        $name . q[ ] . ( join q[ ], map { q["] . $_ . q["] } @arguments ),
+        0, Win32::Process::NORMAL_PRIORITY_CLASS(), q[.] );
+    if ( !$result ) {
+        my $error = Win32::FormatMessage( Win32::GetLastError() );
+        $error =~ s/[\r\n]//smxg;
+        $error =~ s/[.]$//smxg;
+        chomp $error;
+        Carp::croak($error);
+    }
     $self->{_win32_process} = $process;
     return $process->GetProcessID();
 }
@@ -113,6 +121,7 @@ sub _launch_unix {
     my ( $self, @arguments ) = @_;
     my $binary = $self->_binary();
     my $pid;
+    my $dev_null = File::Spec->devnull();
     if ( $pid = fork ) {
     }
     elsif ( defined $pid ) {
@@ -211,7 +220,7 @@ sub _setup_local_connection_to_firefox {
 }
 
 sub _setup_new_profile {
-    my ($self, $profile) = @_;
+    my ( $self, $profile ) = @_;
     my $profile_directory = File::Temp->newdir(
         File::Spec->catdir(
             File::Spec->tmpdir(), 'firefox_marionette_XXXXXXXXXX'
@@ -221,12 +230,11 @@ sub _setup_new_profile {
     $self->{profile_path} =
       File::Spec->catfile( $profile_directory, 'prefs.js' );
     if ($profile) {
-	} else {
-	    my $port = _ANY_PORT();
-		$profile = Firefox::Marionette::Profile->new();
-	    $profile->set_value('marionette.port', $port);
-	}
-    $profile->save($self->{profile_path});
+    }
+    else {
+        $profile = Firefox::Marionette::Profile->new();
+    }
+    $profile->save( $self->{profile_path} );
     return $profile_directory;
 }
 
@@ -730,7 +738,7 @@ sub active_element {
         [ _COMMAND(), $message_id, 'WebDriver:GetActiveElement' ] );
     my $response = $self->_get_response($message_id);
     return Firefox::Marionette::Element->new( $self,
-        %{$response->result()->{value}} );
+        %{ $response->result()->{value} } );
 }
 
 sub uri {
@@ -890,6 +898,15 @@ sub window_handles {
     return @{ $response->result() };
 }
 
+sub close_current_chrome_window_handle {
+    my ($self) = @_;
+    my $message_id = $self->_new_message_id();
+    $self->_send_request(
+        [ _COMMAND(), $message_id, 'WebDriver:CloseChromeWindow' ] );
+    my $response = $self->_get_response($message_id);
+    return @{ $response->result() };
+}
+
 sub css {
     my ( $self, $element, $property_name ) = @_;
     my $message_id = $self->_new_message_id();
@@ -1108,6 +1125,19 @@ sub switch_to_shadow_root {
     return $self;
 }
 
+sub switch_to_window {
+    my ( $self, $window_handle ) = @_;
+    my $message_id = $self->_new_message_id();
+    $self->_send_request(
+        [
+            _COMMAND(), $message_id,
+            'WebDriver:SwitchToWindow', { name => $window_handle }
+        ]
+    );
+    my $response = $self->_get_response($message_id);
+    return $self;
+}
+
 sub switch_to_frame {
     my ( $self, $element ) = @_;
     my $message_id = $self->_new_message_id();
@@ -1139,7 +1169,7 @@ my $x = <<'_DOC_';
 #  "WebDriver:AcceptDialog": GeckoDriver.prototype.acceptDialog,
 #  "WebDriver:AddCookie": GeckoDriver.prototype.addCookie,
 #  "WebDriver:Back": GeckoDriver.prototype.goBack,
-  "WebDriver:CloseChromeWindow": GeckoDriver.prototype.closeChromeWindow,
+#  "WebDriver:CloseChromeWindow": GeckoDriver.prototype.closeChromeWindow,
   "WebDriver:CloseWindow": GeckoDriver.prototype.close,
 #  "WebDriver:DeleteAllCookies": GeckoDriver.prototype.deleteAllCookies,
 #  "WebDriver:DeleteCookie": GeckoDriver.prototype.deleteCookie,
@@ -1194,7 +1224,7 @@ my $x = <<'_DOC_';
 #  "WebDriver:SwitchToFrame": GeckoDriver.prototype.switchToFrame,
 #  "WebDriver:SwitchToParentFrame": GeckoDriver.prototype.switchToParentFrame,
 #  "WebDriver:SwitchToShadowRoot": GeckoDriver.prototype.switchToShadowRoot,
-  "WebDriver:SwitchToWindow": GeckoDriver.prototype.switchToWindow,
+#  "WebDriver:SwitchToWindow": GeckoDriver.prototype.switchToWindow,
 #  "WebDriver:TakeScreenshot": GeckoDriver.prototype.takeScreenshot,
 _DOC_
 
@@ -1293,7 +1323,7 @@ Firefox::Marionette - Automate the Firefox browser with the Marionette protocol
 
 =head1 VERSION
 
-Version 0.10
+Version 0.11
 
 =head1 SYNOPSIS
 
@@ -1326,7 +1356,7 @@ accepts an optional hash as a parameter.  Allowed keys are below;
 
 =item * capabilities - use the supplied L<capabilities|Firefox::Marionette::Capabilities> object, for example to set whether the browser should L<accept insecure certs|Firefox::Marionette::Capabilities#accept_insecure_certs>
 
-=item * profile_name - pick a specific existing profile to automate, rather than creating a new profile.  Note that L<firefox|https://firefox.com> refuses to allow more than one instance of a profile to run at the same time.
+=item * profile_name - pick a specific existing profile to automate, rather than creating a new profile.  Note that L<firefox|https://firefox.com> refuses to allow more than one instance of a profile to run at the same time.  Profile names can be obtained by using the L<Firefox::Marionette::Profile::names()|Firefox::Marionette::Profile#names> method.
 
 =item * profile - create a new profile based on the supplied profile.
 
@@ -1458,6 +1488,10 @@ returns the current active L<frame|Firefox::Marionette::Element> if there is one
 
 accepts an L<elemnet|Firefox::Marionette::Element> as a parameter and switches to it's L<shadow root|https://www.w3.org/TR/shadow-dom/>
 
+=head2 switch_to_window
+
+accepts a window handle (either the result of L<window_handles|Firefox::Marionette#window_handles> or a window name as a parameter and switches focus to this window.
+
 =head2 switch_to_frame
 
 accepts a L<frame|Firefox::Marionette::Element> as a parameter and switches to it within the current window.
@@ -1466,6 +1500,10 @@ accepts a L<frame|Firefox::Marionette::Element> as a parameter and switches to i
 
 set the current browsing context for future commands to the parent of the current browsing context
 
+=head2 close_current_chrome_window_handle
+
+closes the current chrome window (that is the entire window, not just the tabs).  It returns a list of still available chrome window handles. You will need to L<switch_to_window|Firefox::Marionette#switch_to_window> to use another window.
+
 =head2 full_screen
 
 full screens the firefox window. This method returns L<itself|Firefox::Marionette> to aid in chaining methods.
